@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2023 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2023 Laszlo Molnar
+   Copyright (C) 1996-2025 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2025 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -30,7 +30,7 @@
 
 static unsigned hex(uchar c) { return (c & 0xf) + (c > '9' ? 9 : 0); }
 
-static bool update_capacity(unsigned size, unsigned *capacity) {
+static bool grow_capacity(unsigned size, unsigned *capacity) {
     if (size < *capacity)
         return false;
     if (*capacity == 0)
@@ -40,16 +40,22 @@ static bool update_capacity(unsigned size, unsigned *capacity) {
     return true;
 }
 
-static void internal_error(const char *format, ...) attribute_format(1, 2);
-static void internal_error(const char *format, ...) {
-    static char buf[1024];
-    va_list ap;
+template <class T>
+static T **realloc_array(T **array, size_t capacity) may_throw {
+    assert_noexcept(capacity > 0);
+    // NOLINTNEXTLINE(bugprone-multi-level-implicit-pointer-conversion)
+    void *p = realloc(array, mem_size(sizeof(T *), capacity));
+    assert_noexcept(p != nullptr);
+    return static_cast<T **>(p);
+}
 
-    va_start(ap, format);
-    upx_safe_vsnprintf(buf, sizeof(buf), format, ap);
-    va_end(ap);
-
-    throwInternalError(buf);
+template <class T>
+static void free_array(T **array, size_t count) noexcept {
+    for (size_t i = 0; i < count; i++) {
+        T *item = upx::atomic_exchange(&array[i], (T *) nullptr);
+        delete item;
+    }
+    ::free(array); // NOLINT(bugprone-multi-level-implicit-pointer-conversion)
 }
 
 /*************************************************************************
@@ -59,19 +65,19 @@ static void internal_error(const char *format, ...) {
 ElfLinker::Section::Section(const char *n, const void *i, unsigned s, unsigned a)
     : name(nullptr), output(nullptr), size(s), offset(0), p2align(a), next(nullptr) {
     name = strdup(n);
-    assert(name != nullptr);
-    input = malloc(s + 1);
-    assert(input != nullptr);
+    assert_noexcept(name != nullptr);
+    input = ::malloc(s + 1);
+    assert_noexcept(input != nullptr);
     if (s != 0) {
-        assert(i != nullptr);
+        assert_noexcept(i != nullptr);
         memcpy(input, i, s);
     }
     ((char *) input)[s] = 0;
 }
 
 ElfLinker::Section::~Section() noexcept {
-    free(name);
-    free(input);
+    ::free(name);
+    ::free(input);
 }
 
 /*************************************************************************
@@ -81,11 +87,11 @@ ElfLinker::Section::~Section() noexcept {
 ElfLinker::Symbol::Symbol(const char *n, Section *s, upx_uint64_t o)
     : name(nullptr), section(s), offset(o) {
     name = strdup(n);
-    assert(name != nullptr);
-    assert(section != nullptr);
+    assert_noexcept(name != nullptr);
+    assert_noexcept(section != nullptr);
 }
 
-ElfLinker::Symbol::~Symbol() noexcept { free(name); }
+ElfLinker::Symbol::~Symbol() noexcept { ::free(name); }
 
 /*************************************************************************
 // Relocation
@@ -94,7 +100,7 @@ ElfLinker::Symbol::~Symbol() noexcept { free(name); }
 ElfLinker::Relocation::Relocation(const Section *s, unsigned o, const char *t, const Symbol *v,
                                   upx_uint64_t a)
     : section(s), offset(o), type(t), value(v), add(a) {
-    assert(section != nullptr);
+    assert_noexcept(section != nullptr);
 }
 
 /*************************************************************************
@@ -106,17 +112,9 @@ ElfLinker::ElfLinker(const N_BELE_RTP::AbstractPolicy *b) noexcept : bele(b) {}
 ElfLinker::~ElfLinker() noexcept {
     delete[] input;
     delete[] output;
-
-    unsigned ic;
-    for (ic = 0; ic < nsections; ic++)
-        delete sections[ic];
-    free(sections);
-    for (ic = 0; ic < nsymbols; ic++)
-        delete symbols[ic];
-    free(symbols);
-    for (ic = 0; ic < nrelocations; ic++)
-        delete relocations[ic];
-    free(relocations);
+    free_array(sections, nsections);
+    free_array(symbols, nsymbols);
+    free_array(relocations, nrelocations);
 }
 
 void ElfLinker::init(const void *pdata_v, int plen, unsigned pxtra) {
@@ -156,7 +154,7 @@ void ElfLinker::init(const void *pdata_v, int plen, unsigned pxtra) {
     input[inputlen] = 0; // NUL terminate
 
     output_capacity = (inputlen ? (inputlen + pxtra) : 0x4000);
-    assert(output_capacity <= (1 << 16)); // LE16 l_info.l_size
+    assert(output_capacity < (1 << 16)); // LE16 l_info.l_size
     output = New(byte, output_capacity);
     outputlen = 0;
     NO_printf("\nElfLinker::init %d @%p\n", output_capacity, output);
@@ -186,8 +184,9 @@ void ElfLinker::init(const void *pdata_v, int plen, unsigned pxtra) {
 }
 
 void ElfLinker::preprocessSections(char *start, char const *end) {
+    assert_noexcept(nsections == 0);
     char *nextl;
-    for (nsections = 0; start < end; start = 1 + nextl) {
+    for (; start < end; start = 1 + nextl) {
         nextl = strchr(start, '\n');
         assert(nextl != nullptr);
         *nextl = '\0'; // a record is a line
@@ -207,8 +206,9 @@ void ElfLinker::preprocessSections(char *start, char const *end) {
 }
 
 void ElfLinker::preprocessSymbols(char *start, char const *end) {
+    assert_noexcept(nsymbols == 0);
     char *nextl;
-    for (nsymbols = 0; start < end; start = 1 + nextl) {
+    for (; start < end; start = 1 + nextl) {
         nextl = strchr(start, '\n');
         assert(nextl != nullptr);
         *nextl = '\0'; // a record is a line
@@ -227,7 +227,7 @@ void ElfLinker::preprocessSymbols(char *start, char const *end) {
         else if (sscanf(start, "%x%*8c %1023s %*x %1023s", &offset, section, symbol) == 3)
 #else
         // work around broken scanf() implementations
-        // http://bugs.winehq.org/show_bug.cgi?id=10401 (fixed in Wine 0.9.58)
+        // https://bugs.winehq.org/show_bug.cgi?id=10401 (fixed in Wine 0.9.58)
         else if (sscanf(start, "%x%*c%*c%*c%*c%*c%*c%*c%*c %1023s %*x %1023s", &offset, section,
                         symbol) == 3)
 #endif
@@ -243,9 +243,10 @@ void ElfLinker::preprocessSymbols(char *start, char const *end) {
 }
 
 void ElfLinker::preprocessRelocations(char *start, char const *end) {
+    assert_noexcept(nrelocations == 0);
     Section *section = nullptr;
     char *nextl;
-    for (nrelocations = 0; start < end; start = 1 + nextl) {
+    for (; start < end; start = 1 + nextl) {
         nextl = strchr(start, '\n');
         assert(nextl != nullptr);
         *nextl = '\0'; // a record is a line
@@ -302,7 +303,7 @@ ElfLinker::Section *ElfLinker::findSection(const char *name, bool fatal) const {
         if (strcmp(sections[ic]->name, name) == 0)
             return sections[ic];
     if (fatal)
-        internal_error("unknown section %s\n", name);
+        throwInternalError("unknown section %s\n", name);
     return nullptr;
 }
 
@@ -311,7 +312,7 @@ ElfLinker::Symbol *ElfLinker::findSymbol(const char *name, bool fatal) const {
         if (strcmp(symbols[ic]->name, name) == 0)
             return symbols[ic];
     if (fatal)
-        internal_error("unknown symbol %s\n", name);
+        throwInternalError("unknown symbol %s\n", name);
     return nullptr;
 }
 
@@ -320,14 +321,11 @@ ElfLinker::Section *ElfLinker::addSection(const char *sname, const void *sdata,
     NO_printf("addSection: %s len=%d align=%d\n", sname, slen, p2align);
     if (!sdata && (!strcmp("ABS*", sname) || !strcmp("UND*", sname)))
         return nullptr;
-    if (update_capacity(nsections, &nsections_capacity))
-        sections =
-            static_cast<Section **>(realloc(sections, nsections_capacity * sizeof(Section *)));
-    assert(sections);
-    assert(sname);
-    assert(sname[0]);
+    assert(sname && sname[0]);
     assert(sname[strlen(sname) - 1] != ':');
     assert(findSection(sname, false) == nullptr);
+    if (grow_capacity(nsections, &nsections_capacity))
+        sections = realloc_array(sections, nsections_capacity);
     Section *sec = new Section(sname, sdata, slen, p2align);
     sec->sort_id = nsections;
     sections[nsections++] = sec;
@@ -337,13 +335,11 @@ ElfLinker::Section *ElfLinker::addSection(const char *sname, const void *sdata,
 ElfLinker::Symbol *ElfLinker::addSymbol(const char *name, const char *section,
                                         upx_uint64_t offset) {
     NO_printf("addSymbol: %s %s 0x%llx\n", name, section, offset);
-    if (update_capacity(nsymbols, &nsymbols_capacity))
-        symbols = static_cast<Symbol **>(realloc(symbols, nsymbols_capacity * sizeof(Symbol *)));
-    assert(symbols != nullptr);
-    assert(name);
-    assert(name[0]);
+    assert(name && name[0]);
     assert(name[strlen(name) - 1] != ':');
     assert(findSymbol(name, false) == nullptr);
+    if (grow_capacity(nsymbols, &nsymbols_capacity))
+        symbols = realloc_array(symbols, nsymbols_capacity);
     Symbol *sym = new Symbol(name, findSection(section), offset);
     symbols[nsymbols++] = sym;
     return sym;
@@ -351,19 +347,16 @@ ElfLinker::Symbol *ElfLinker::addSymbol(const char *name, const char *section,
 
 ElfLinker::Relocation *ElfLinker::addRelocation(const char *section, unsigned off, const char *type,
                                                 const char *symbol, upx_uint64_t add) {
-    if (update_capacity(nrelocations, &nrelocations_capacity))
-        relocations = static_cast<Relocation **>(
-            realloc(relocations, (nrelocations_capacity) * sizeof(Relocation *)));
-    assert(relocations != nullptr);
+    if (grow_capacity(nrelocations, &nrelocations_capacity))
+        relocations = realloc_array(relocations, nrelocations_capacity);
     Relocation *rel = new Relocation(findSection(section), off, type, findSymbol(symbol), add);
     relocations[nrelocations++] = rel;
     return rel;
 }
 
 #if 0
-void ElfLinker::setLoaderAlignOffset(int phase)
-{
-    //assert(phase & 0);
+void ElfLinker::setLoaderAlignOffset(int phase) {
+    // assert(phase & 0);
     NO_printf("\nFIXME: ElfLinker::setLoaderAlignOffset %d\n", phase);
 }
 #endif
@@ -425,7 +418,7 @@ int ElfLinker::addLoader(const char *sname) {
         }
         sect += strlen(sect) + 1;
     }
-    free(begin);
+    ::free(begin);
     return outputlen;
 }
 
@@ -474,10 +467,10 @@ void ElfLinker::relocate() {
             value = rel->value->offset;
         } else if (strcmp(rel->value->section->name, "*UND*") == 0 &&
                    rel->value->offset == 0xdeaddead)
-            internal_error("undefined symbol '%s' referenced\n", rel->value->name);
+            throwInternalError("undefined symbol '%s' referenced\n", rel->value->name);
         else if (rel->value->section->output == nullptr)
-            internal_error("can not apply reloc '%s:%x' without section '%s'\n", rel->section->name,
-                           rel->offset, rel->value->section->name);
+            throwInternalError("can not apply reloc '%s:%x' without section '%s'\n",
+                               rel->section->name, rel->offset, rel->value->section->name);
         else {
             value = rel->value->section->offset + rel->value->offset + rel->add;
         }
@@ -495,7 +488,7 @@ void ElfLinker::relocate() {
 void ElfLinker::defineSymbol(const char *name, upx_uint64_t value) {
     Symbol *symbol = findSymbol(name);
     if (strcmp(symbol->section->name, "*ABS*") == 0)
-        internal_error("defineSymbol: symbol '%s' is *ABS*\n", name);
+        throwInternalError("defineSymbol: symbol '%s' is *ABS*\n", name);
     else if (strcmp(symbol->section->name, "*UND*") == 0) // for undefined symbols
         symbol->offset = value;
     else if (strcmp(symbol->section->name, name) == 0) // for sections
@@ -506,7 +499,7 @@ void ElfLinker::defineSymbol(const char *name, upx_uint64_t value) {
             value += section->size;
         }
     } else
-        internal_error("defineSymbol: symbol '%s' already defined\n", name);
+        throwInternalError("defineSymbol: symbol '%s' already defined\n", name);
 }
 
 // debugging support
@@ -553,7 +546,7 @@ void ElfLinker::alignWithByte(unsigned len, byte b) {
 }
 
 void ElfLinker::relocate1(const Relocation *rel, byte *, upx_uint64_t, const char *) {
-    internal_error("unknown relocation type '%s\n'", rel->type);
+    throwInternalError("unknown relocation type '%s\n'", rel->type);
 }
 
 /*************************************************************************
@@ -563,14 +556,13 @@ void ElfLinker::relocate1(const Relocation *rel, byte *, upx_uint64_t, const cha
 **************************************************************************/
 
 #if 0 // FIXME
-static void check8(const Relocation *rel, const byte *location, int v, int d)
-{
+static void check8(const Relocation *rel, const byte *location, int v, int d) {
     if (v < -128 || v > 127)
-        internal_error("value out of range (%d) in reloc %s:%x\n",
-                       v, rel->section->name, rel->offset);
+        throwInternalError("value out of range (%d) in reloc %s:%x\n", v, rel->section->name,
+                           rel->offset);
     if (d < -128 || d > 127)
-        internal_error("displacement target out of range (%d) in reloc %s:%x\n",
-                       v, rel->section->name, rel->offset);
+        throwInternalError("displacement target out of range (%d) in reloc %s:%x\n", v,
+                           rel->section->name, rel->offset);
 }
 #endif
 
@@ -592,10 +584,10 @@ void ElfLinkerAMD64::relocate1(const Relocation *rel, byte *location, upx_uint64
     }
 
     if (strcmp(type, "8") == 0) {
-        int displ = (signed char) *location + (int) value;
+        int displ = (upx_int8_t) *location + (int) value;
         if (range_check && (displ < -128 || displ > 127))
-            internal_error("target out of range (%d) in reloc %s:%x\n", displ, rel->section->name,
-                           rel->offset);
+            throwInternalError("target out of range (%d) in reloc %s:%x\n", displ,
+                               rel->section->name, rel->offset);
         *location += value;
     } else if (strcmp(type, "16") == 0)
         set_le16(location, get_le16(location) + value);
@@ -688,7 +680,7 @@ void ElfLinkerArm64LE::relocate1(const Relocation *rel, byte *location, upx_uint
         upx_uint32_t const m19 = ~(~0u << 19);
         upx_uint32_t w = get_le32(location);
         set_le32(location, (w & ~(m19 << 5)) | ((((w >> 5) + (value >> 2)) & m19) << 5));
-    } else if (!strcmp(type, "CALL26")) {
+    } else if (!strcmp(type, "CALL26") || !strcmp(type, "JUMP26")) {
         value -= rel->section->offset + rel->offset;
         upx_uint32_t const m26 = ~(~0u << 26);
         upx_uint32_t w = get_le32(location);
@@ -800,12 +792,12 @@ void ElfLinkerPpc32::relocate1(const Relocation *rel, byte *location, upx_uint64
     // Note that original (*location).displ is ignored.
     if (strcmp(type, "24") == 0) {
         if (3 & value)
-            internal_error("unaligned word displacement");
+            throwInternalError("unaligned word displacement");
         // FIXME: displacement overflow?
         set_be32(location, (0xfc000003 & get_be32(location)) + (0x03fffffc & value));
     } else if (strcmp(type, "14") == 0) {
         if (3 & value)
-            internal_error("unaligned word displacement");
+            throwInternalError("unaligned word displacement");
         // FIXME: displacement overflow?
         set_be32(location, (0xffff0003 & get_be32(location)) + (0x0000fffc & value));
     } else
@@ -835,18 +827,24 @@ void ElfLinkerPpc64le::relocate1(const Relocation *rel, byte *location, upx_uint
     /* value will hold relative displacement */
     value -= rel->section->offset + rel->offset;
 
-    if (strcmp(type, "8") == 0) {
-        int displ = (signed char) *location + (int) value;
+    if (strncmp(type, "14", 2) == 0) { // for "14" and "14S"
+        if (3 & value)
+            throwInternalError("unaligned word displacement");
+        // FIXME: displacement overflow?
+        set_le32(location, (0xffff0003 & get_le32(location)) + (0x0000fffc & value));
+    } else if (strncmp(type, "24", 2) == 0) { // for "24" and "24S"
+        if (3 & value)
+            throwInternalError("unaligned word displacement");
+        // FIXME: displacement overflow?
+        set_le32(location, (0xfc000003 & get_le32(location)) + (0x03fffffc & value));
+    } else if (strcmp(type, "8") == 0) {
+        int displ = (upx_int8_t) *location + (int) value;
         if (range_check && (displ < -128 || displ > 127))
-            internal_error("target out of range (%d) in reloc %s:%x\n", displ, rel->section->name,
-                           rel->offset);
+            throwInternalError("target out of range (%d) in reloc %s:%x\n", displ,
+                               rel->section->name, rel->offset);
         *location += value;
-    } else if (strncmp(type, "14", 2) == 0) // for "14" and "14S"
+    } else if (strcmp(type, "16") == 0)
         set_le16(location, get_le16(location) + value);
-    else if (strcmp(type, "16") == 0)
-        set_le16(location, get_le16(location) + value);
-    else if (strncmp(type, "24", 2) == 0) // for "24" and "24S"
-        set_le24(location, get_le24(location) + value);
     else if (strncmp(type, "32", 2) == 0) // for "32" and "32S"
         set_le32(location, get_le32(location) + value);
     else if (strcmp(type, "64") == 0)
@@ -869,18 +867,38 @@ void ElfLinkerPpc64::relocate1(const Relocation *rel, byte *location, upx_uint64
         return super::relocate1(rel, location, value, type);
     type += 11;
 
+    bool range_check = false;
     if (strncmp(type, "PC", 2) == 0) {
         type += 2;
+        range_check = true;
     }
 
     // We have "R_PPC64_REL" or "R_PPC64_RELPC" as a prefix.
     /* value will hold relative displacement */
     value -= rel->section->offset + rel->offset;
 
-    if (strncmp(type, "14", 2) == 0) // for "14" and "14S"
-        set_be16(2 + location, get_be16(2 + location) + value);
-    else if (strncmp(type, "24", 2) == 0) // for "24" and "24S"
-        set_be24(1 + location, get_be24(1 + location) + value);
+    if (strncmp(type, "14", 2) == 0) { // for "14" and "14S"
+        if (3 & value)
+            throwInternalError("unaligned word displacement");
+        // FIXME: displacement overflow?
+        set_be32(location, (0xffff0003 & get_be32(location)) + (0x0000fffc & value));
+    } else if (strncmp(type, "24", 2) == 0) { // for "24" and "24S"
+        if (3 & value)
+            throwInternalError("unaligned word displacement");
+        // FIXME: displacement overflow?
+        set_be32(location, (0xfc000003 & get_be32(location)) + (0x03fffffc & value));
+    } else if (strcmp(type, "8") == 0) {
+        int displ = (upx_int8_t) *location + (int) value;
+        if (range_check && (displ < -128 || displ > 127))
+            throwInternalError("target out of range (%d) in reloc %s:%x\n", displ,
+                               rel->section->name, rel->offset);
+        *location += value;
+    } else if (strcmp(type, "16") == 0)
+        set_be16(location, get_be16(location) + value);
+    else if (strncmp(type, "32", 2) == 0) // for "32" and "32S"
+        set_be32(location, get_be32(location) + value);
+    else if (strcmp(type, "64") == 0)
+        set_be64(location, get_be64(location) + value);
     else
         super::relocate1(rel, location, value, type);
 }
@@ -899,10 +917,10 @@ void ElfLinkerX86::relocate1(const Relocation *rel, byte *location, upx_uint64_t
     }
 
     if (strcmp(type, "8") == 0) {
-        int displ = (signed char) *location + (int) value;
+        int displ = (upx_int8_t) *location + (int) value;
         if (range_check && (displ < -128 || displ > 127))
-            internal_error("target out of range (%d,%d,%llu) in reloc %s:%x\n", displ, *location,
-                           value, rel->section->name, rel->offset);
+            throwInternalError("target out of range (%d,%d,%llu) in reloc %s:%x\n", displ,
+                               *location, value, rel->section->name, rel->offset);
         *location += value;
     } else if (strcmp(type, "16") == 0)
         set_le16(location, get_le16(location) + value);
